import java.util.Random;

public class RumbleBot extends SmartBot {

    private int tilesToMove;

    private int escapeMove;

    private final Random random;

    private final int[] leftVectorIndizes = new int[] {36,37,38,39};
    private final int[] rightVectorIndizes = new int[] {23,24,25,26};
    private final int[] frontVectorIndizes = new int[] {4,13,22,31};

    protected RumbleBot(String[] args) {
        super(args);
        this.random = new Random();
        this.escapeMove = 0;
        this.tilesToMove = random.nextInt(5, 15);
    }

    public static void main(String[] args) {
        Bot bot = new RumbleBot(args);
        bot.run();
    }

    @Override
    protected char nextMove(View view) throws Exception {
        char move = FORWARD;

//        if (tilesToMove >= tilesMoved) {
//            tilesMoved = 0;
//            tilesToMove = random.nextInt(5, 15);
//            move = directionBias;
//        }

        if (escapeMove != 0) {
            escapeMove--;
            return FORWARD;
        }

        // Check for enemies
        Enemy enemy = evaluateEnemy(view);
        if (enemy != null) {
            move = calculateOptimalMove(view, enemy);
            escapeMove = 3;
        }

        if (directionBlocked(view,FORWARD)) {
            if (directionBlocked(view,RIGHT)) {
                move = LEFT;
            } else {
                move = RIGHT;
            }
        }

        return move;
    }

    private Enemy evaluateEnemy(View view) {
        int enemyPos;
        if (view.data.contains("^")) {
            enemyPos = view.data.indexOf('^');
        } else if (view.data.contains("v")) {
            enemyPos = view.data.indexOf('v');
        } else if (view.data.contains("<")) {
            enemyPos = view.data.indexOf('<');
        } else if (view.data.contains(">")) {
            enemyPos = view.data.indexOf('>');
        } else {
            return null;
        }
        return new Enemy(enemyPos + 1, view.data.charAt(enemyPos));
    }

    public boolean directionBlocked(View view, char direction) {
        if (direction == FORWARD) {
            for (int i = 4; i < 32; i+=9) {
                if (view.data.charAt(i) == 'X') {
                    return true;
                }
            }
        } else if (direction == LEFT) {
            for (int i = 36; i < 40; i++) {
                if (view.data.charAt(i) == 'X') {
                    return true;
                }
            }
        } else if (direction == RIGHT) {
            for (int i = 41; i < 45; i++) {
                if (view.data.charAt(i) == 'X') {
                    return true;
                }
            }
        }
        return false;
    }

    private char calculateOptimalMove(View view, Enemy enemy) {
        int pos = enemy.position;
        char dir = enemy.direction;


        // Top Sector
        if (pos <= 18) {
            // Left Half
            if (pos % 9 < 5) {
                return !directionBlocked(view,RIGHT) ? RIGHT : SHOOT;
            }

            // Right Half
            else if (pos % 9 > 5) {
                return !directionBlocked(view,LEFT) ? LEFT : SHOOT;
            }
            // Center
            else {
                return SHOOT;
            }
        }

        // Bottom Sector
        if (pos >= 24) {
            // Left or Right Half
            if (pos % 9 != 5) {
                return !directionBlocked(view,FORWARD) ? FORWARD : randomDirection(view);
            }
            // Center
            else {
                return randomDirection(view);
            }
        }

        // Left or Right
        return randomDirection(view);
    }

    private char randomDirection(View view) {
        int rand = random.nextInt(0, 1);
        char dir = rand == 0 ? '<' : '>';
        return !directionBlocked(view,dir) ? dir : flipDirection(dir);
    }

    private char flipDirection(char direction) {
        return switch (direction) {
            case RIGHT -> LEFT;
            case LEFT -> RIGHT;
            case FORWARD -> BACKWARD;
            default -> FORWARD;
        };
    }

    private static class Enemy {
        final int position;
        final char direction;

        public Enemy(int position, char direction) {
            this.position = position;
            this.direction = direction;
        }

    }

}
